package com.jln.common.entity;

import com.jln.common.enums.ResultCodeEnum;

/**
 * @author 86180
 * @version : Result, v 0.1 2020/1/17 11:08
 */
public class Result<T> {


   private String msg;
   private T data;
   private int code;

   public String getMsg() {
      return msg;
   }


   public Result<T> setMsg(String msg) {
      this.msg = msg;
      return this;
   }

   public T getData() {
      return data;
   }

   public Result<T> setData(T data) {
      this.data = data;
      return this;
   }

   public int getCode() {
      return code;
   }

   public Result<T> setCode(int code) {
      this.code = code;
      return this;
   }

   public Result(T data) {
      this(ResultCodeEnum.SUCCESS, data);
   }

   public Result() {
   }

   public Result(ResultCodeEnum resultCode, T data) {
      this.code = resultCode.getCode();
      this.msg = resultCode.getMsg();
      this.data = data;
   }

   /**
    * 失败且带返回信息
    *
    * @param resultCode
    * @return
    */
   public Result toFailure(ResultCodeEnum resultCode) {
      this.setCode(resultCode.getCode());
      this.setMsg(resultCode.getMsg());
      this.setData((T) resultCode.getMsg());
      return this;

   }

   /**
    * 成功且带返回信息
    * @param data
    * @return
    */
   public Result<T> toSuccess(T data){
      this.setCode(ResultCodeEnum.SUCCESS.getCode());
      this.setMsg(ResultCodeEnum.SUCCESS.getMsg());
      this.setData(data);
      return this;
   }
   /**
    * 成功且不带带返回信息
    * @return
    */
   public Result<T> toSuccess(){
      this.setCode(ResultCodeEnum.SUCCESS.getCode());
      this.setMsg(ResultCodeEnum.SUCCESS.getMsg());
      this.setData(null);
      return this;
   }
   /**
    * 成功且不带带返回信息
    * @return
    */
   public Result<T> toSuccess(ResultCodeEnum resultCode){
      this.setCode(resultCode.getCode());
      this.setMsg(resultCode.getMsg());
      this.setData(null);
      return this;
   }

   /**
    * 失败且返回消息由外部指定
    *
    * @param
    * @return
    */
   public Result toFailure(String msg) {
      this.setCode(-1);
      this.setMsg(msg);
      return this;

   }


   /**
    * 判断请求是否成功
    *
    * @return
    */
   public boolean confirmSuccess() {
      if (this.code == 0 || this.code == 200) {
         return true;
      }
      return false;
   }

}
