package com.jln.common.exception;

import com.alibaba.fastjson.JSONObject;
import com.jln.common.entity.ResponseResult;
import com.jln.common.utils.DateUtils;
import com.jln.common.utils.HttpHelper;
import com.jln.common.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 三一中控接口统一异常处理
 */
@RestControllerAdvice(basePackages = "com.jln.system.web.action.syzk.controller")
public class GlobalSyzkExceptionHandler {
   private static final Logger logger = LoggerFactory.getLogger(GlobalSyzkExceptionHandler.class);

   /**
    * 未知运行异常Exception
    */
   @ExceptionHandler({Exception.class})
   @ResponseStatus(HttpStatus.BAD_REQUEST)
   public ResponseResult ExceptionHandler(Exception e, HttpServletRequest request) {
      logger.error("未知异常：" + e.getMessage());
      try {
         String bodyString = HttpHelper.getBodyString(request);
         JSONObject jsonObject = JSONObject.parseObject(bodyString);
         if (null != jsonObject) {
            return new ResponseResult().toFailure(jsonObject.getString("request_uuid"), DateUtils.getDateTime(), e.getMessage());
         }
      } catch (IOException exception) {
         logger.error("获取参数request_uuid异常：" + exception.getMessage());
      }
      return new ResponseResult().toFailure("", DateUtils.getDateTime(), e.getMessage());
   }


   /**
    * 方法参数错误异常
    *
    * @param e
    * @return
    */
   @ExceptionHandler(MethodArgumentNotValidException.class)
   @ResponseStatus(HttpStatus.BAD_REQUEST)
   public ResponseResult MethodArgumentNotValidExceptionHandler(MethodArgumentNotValidException e, HttpServletRequest request) {
      logger.error("方法参数错误异常：" + e.getMessage());
      List<String> list = new ArrayList<>();
      // 从异常对象中拿到ObjectError对象
      if (!e.getBindingResult().getAllErrors().isEmpty()) {
         for (ObjectError error : e.getBindingResult().getAllErrors()) {
            list.add(error.getDefaultMessage().toString());
         }
      }
      try {
         String bodyString = HttpHelper.getBodyString(request);
         JSONObject jsonObject = JSONObject.parseObject(bodyString);
         if (null != jsonObject) {
            return new ResponseResult().toFailure(jsonObject.getString("request_uuid"), DateUtils.getDateTime(), StringUtils.join(list, ","));
         }
      } catch (IOException exception) {
         logger.error("获取参数request_uuid异常：" + exception.getMessage());
      }
      return new ResponseResult().toFailure("", DateUtils.getDateTime(), StringUtils.join(list, ","));
   }

}
