package com.jln.common.utils;



import org.springframework.util.ResourceUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 上传文件的工具类
 */
public class FileUtil {
    private String imgUrL;  //服务器储存上传图片地址的前缀
    private String getPath;
    private String path;         //存放的路径,不带具体项目名称
    private String QRNAME;      //生成二维码文件名

    public FileUtil(){}

    public FileUtil(HttpServletRequest request, String imgUrL) {
        this.QRNAME=BaseUtil.getUuidRp()+".png";
        try {
            File fileURL=new File(getJarRootPath(),imgUrL);
            this.path = fileURL.getAbsolutePath()+File.separator;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        this.imgUrL=imgUrL;
        this.getPath=request.getScheme()+"://"+request.getServerName()+":"
                +request.getServerPort() + request.getContextPath();
    }

    /**
     * 截取文件类型后缀如:jpg，png
     * @return
     */
    private String IsFile(MultipartFile file){
        String oriName=null;
        if(!file.isEmpty()){
            oriName = file.getOriginalFilename();
            //获取出文件的后缀
            int index=oriName.lastIndexOf(".");
            oriName=oriName.substring(index);
        }
        return oriName;
    }

    /**
     * 保存文件工具类，返回文件网络地址
     * @return
     */
    public String uploadFile(MultipartFile file){
        String ImgNmae=BaseUtil.getUUID();
        String paths=path+ ImgNmae+IsFile(file);
       // System.out.println(paths);
//        String updateURL=getPath+imgUrL+"/"+ImgNmae+IsFile(file);
        String updateURL=imgUrL+"/"+ImgNmae+IsFile(file);
//        String updateURL=paths;
        try {
            file.transferTo(new File(paths));
            return updateURL;
        } catch (IOException e) {
            e.printStackTrace();
            //如果文件夹不存在,则创建一个文件夹
            File file2=new File(path);
            file2.mkdirs();
            try {
                file.transferTo(new File(paths));
                return updateURL;
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 批量上传文件
     */
    public List<String> uploadFile(MultipartFile[] files){
        List<String> ImgUrls=new ArrayList<>();
        for (MultipartFile file : files) {
            String file1 = uploadFile(file);
            ImgUrls.add(file1);
        }
        return ImgUrls;
    }

    /**
     * 删除文件,
     * @param URL
     * @return
     */
    public Boolean deletFile(String URL){
        URL = replacementPath(URL);
        boolean delete = false;
        File file=new File(URL);
        if(file.exists()){
            delete = file.delete();
        }
        return delete;
    }

    /**
     * 修改图片的方法
     * @param updateUrl 数据库中路径
     * @param file   文件对象
     * @return 返回替换结果
     */
    public void updateFile(String updateUrl, MultipartFile file) throws IOException {
        String s = replacementPath(updateUrl);      //将路径替换成当前项目的物理路径
        file.transferTo(new File(s));
    }


    /**
     * 将数据库中的路径替换成实际项目中的路径
     * @return
     */
    public String replacementPath(String dataURL){
        int index = dataURL.lastIndexOf("/");
       if(-1==index){
           index=dataURL.lastIndexOf("\\");
       }
        return path+dataURL.substring(++index);
    }


    /**
     * 处理路径的方法
     * @return
     * @throws FileNotFoundException
     */
    public String getJarRootPath() throws FileNotFoundException {

        String path = ResourceUtils.getURL("classpath:").getPath();
        File rootFile = new File(path);
        if(!rootFile.exists()) {
            rootFile = new File("");
        }

        return rootFile.getAbsolutePath();
    }

    /**
     * 判断后缀是否一致,如果一致就执行替换方法
     * @param updateUrl
     * @param file
     * @return
     */
    public Boolean juitSuffix(String updateUrl, MultipartFile file){
        String Suffix1 = IsFile(file);
        int index1 = updateUrl.lastIndexOf(".");
        updateUrl=updateUrl.substring(index1);
        return Suffix1.equals(updateUrl);
    }

    /**
     * 获取二维码对应在项目里面的路径
     * @return
     */
    public String getPath(){
        //创建当前文件
        String fileURL=this.path+QRNAME;
        File file = new File(path);
        file.mkdirs();
        return fileURL;
    }

    /**
     * 获取项目路径
     * @return
     */
    public String getGetPath(){
        return this.getPath+imgUrL+"/"+QRNAME;
    }

    /**
     * 复制单个文件
     * @param srcFileName 待复制的文件名
     * @param descFileName 目标文件名
     * @param coverlay 如果目标文件已存在，是否覆盖
     * @return 如果复制成功，则返回true，否则返回false
     */
    public boolean copyFile(String srcFileName,
                                        String descFileName, boolean coverlay) {
        File srcFile = new File(srcFileName);
        // 判断源文件是否存在
        if (!srcFile.exists()) {
            return false;
        }
        // 判断源文件是否是合法的文件
        else if (!srcFile.isFile()) {
            return false;
        }
        File descFile = new File(descFileName);
        // 判断目标文件是否存在
        if (descFile.exists()) {
            // 如果目标文件存在，并且允许覆盖
            if (coverlay) {
                if (!deletFile(descFileName)) {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            if (!descFile.getParentFile().exists()) {
                // 如果目标文件所在的目录不存在，则创建目录
                // 创建目标文件所在的目录
                if (!descFile.getParentFile().mkdirs()) {
                    return false;
                }
            }
        }

        // 准备复制文件
        // 读取的位数
        int readByte = 0;
        InputStream ins = null;
        OutputStream outs = null;
        try {
            // 打开源文件
            ins = new FileInputStream(srcFile);
            // 打开目标文件的输出流
            outs = new FileOutputStream(descFile);
            byte[] buf = new byte[1024];
            // 一次读取1024个字节，当readByte为-1时表示文件已经读取完毕
            while ((readByte = ins.read(buf)) != -1) {
                // 将读取的字节流写入到输出流
                outs.write(buf, 0, readByte);
            }
            return true;
        } catch (Exception e) {
            return false;
        } finally {
            // 关闭输入输出流，首先关闭输出流，然后再关闭输入流
            if (outs != null) {
                try {
                    outs.close();
                } catch (IOException oute) {
                    oute.printStackTrace();
                }
            }
            if (ins != null) {
                try {
                    ins.close();
                } catch (IOException ine) {
                    ine.printStackTrace();
                }
            }
        }
    }

}
