package com.jln.common.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TreeUtils {

    protected TreeUtils() {

    }

    public static <T> Tree<T> build(List<Tree<T>> nodes) {
        if (nodes == null) {
            return null;
        }
        List<Tree<T>> topNodes = new ArrayList<>();
        nodes.forEach(children -> {
            String pid = children.getParentId();
            if (pid == null || "00000000-00000000-00000000-000000000".equals(pid)) {
                topNodes.add(children);
                return;
            }
            for (Tree<T> parent : nodes) {
                String id = parent.getId();
                if (id != null && id.equals(pid)) {
                    parent.getChildren().add(children);
                    children.setHasParent(true);
                    parent.setChildren(true);
                    return;
                }
            }
        });

        Tree<T> root = new Tree<>();
        root.setId("00000000-00000000-00000000-000000000");
        root.setParentId("");
        root.setHasParent(false);
        root.setChildren(true);
        root.setChecked(true);
        root.setChildren(topNodes);
        root.setLabel("根节点");
        Map<String, Object> state = new HashMap<>(16);
        state.put("opened", true);
        root.setState(state);
        return root;
    }

    public static <T> List<Tree<T>> buildList(List<Tree<T>> nodes, String idParam) {
        if (nodes == null) {
            return new ArrayList<>();
        }
        List<Tree<T>> topNodes = new ArrayList<>();
        nodes.forEach(children -> {
            String pid = children.getParentId();
            if (pid == null || idParam.equals(pid)) {
                topNodes.add(children);
                return;
            }
            nodes.forEach(parent -> {
                String id = parent.getId();
                if (id != null && id.equals(pid)) {
                    parent.getChildren().add(children);
                    children.setHasParent(true);
                    parent.setChildren(true);
                }
            });
        });
        return topNodes;
    }

    /*public static List<Menu2> menuTree(List<Menu2> menus) {
        List<Menu2> nodeTrees = new ArrayList<>();
        Map<Integer, Menu2> sysMenuMap = menus.stream().collect(Collectors.toMap(Menu2::getId, sysMenu -> sysMenu));
        for (Integer id : sysMenuMap.keySet()) {
            Menu2 sysMenu = sysMenuMap.get(id);
            Integer parent_id = sysMenu.getParent_id();
            if (parent_id == null || parent_id == 0 || !sysMenuMap.containsKey(parent_id)) {
                nodeTrees.add(sysMenu);
            } else {
                Menu2 parentTreeNode = sysMenuMap.get(parent_id);
                if (parentTreeNode.getChildren() == null) {
                    parentTreeNode.setChildren(new ArrayList<>());
                }
                parentTreeNode.getChildren().add(sysMenu);
            }
        }
        //取根节点的树
        nodeTrees = nodeTrees.stream().filter(sysMenu -> sysMenu.getParent_id() == 0L).collect(Collectors.toList());
        return nodeTrees;
    }*/
}