package com.jln.system.web.action.mqtt;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.jln.system.web.action.mqtt.pojo.dto.JsonDataDto;
import com.jln.system.web.action.mqtt.pojo.vo.StaticData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

@Service
@PropertySource("classpath:application.properties")
public class MqttStartClient {

    @Value("${redis.separate}")
    private Integer separate;

    @Value("${public.flag}")
    private Boolean flag;

    @Autowired
    private Base64decrypt base64decrypt;

    /**
     * 解析sk文件并且放入map集合中固定的
     *
     * @param skFileUrl
     * @return
     */
    public Integer resolveSk(String skFileUrl) {
        String skFileValue = readSkFileToList(skFileUrl);
        JSONObject jsonObject = JSONObject.parseObject(skFileValue);
        return jsonObject.getInteger("version");
    }


    /**
     * 获取证书sk解析后的码
     *
     * @return
     */
    public String readSkFileToList(String skFileUrl) {
        List<String> filelist = new ArrayList();
        // if (flag) {  //true读取jar包同级的路径
        //   filelist = readJarSkFile(skFileUrl);
        // } else {  //false读取resources下的路径
        filelist = readResourcesSkFile(skFileUrl);
        // }
        String info = "";

        if (filelist.isEmpty()) {
            return null;
        }
        // 遍历listStr集合
        for (String data : filelist) {
            // +换行
            info = info + data;
        }
        Base64.Decoder decoder = Base64.getDecoder();
        byte[] bytes = decoder.decode(info);
        return new String(bytes);
    }

    /**
     * 解析txt文件
     *
     * @param txtFileUrl
     * @return
     */
    public Map<String, Object> readTxtFileToMap(String txtFileUrl) {
        HashMap<String, Object> sites = new HashMap<>();
        List<String> txtFilelist = new ArrayList();
        if (flag) {
            txtFilelist = readJarSkFile(txtFileUrl);
        } else {
            txtFilelist = readResourcesSkFile(txtFileUrl);
        }
        if (txtFilelist.isEmpty()) {
            return sites;
        }
        for (String data : txtFilelist) {
            String[] split = data.split("=");
            if (split.length == 2) {
                sites.put(split[0], split[1]);
            } else {
                sites.put(split[0], "");
            }

        }
        return sites;
    }


    public List<String> readResourcesSkFile(String skFileUrl) {
        List<String> listStr = new ArrayList<>();
        BufferedReader br;
        String str;
        try {
            ClassPathResource cpr = new ClassPathResource(skFileUrl);
            InputStream is = cpr.getInputStream();

            br = new BufferedReader(new InputStreamReader(is, "UTF-8"));
            while ((str = br.readLine()) != null) {
                listStr.add(str);
            }
        } catch (Exception e) {
            return listStr;
        }
        return listStr;
    }

    public List<String> readJarSkFile(String skFileUrl) {
        List<String> listStr = new ArrayList<>();
        String url = System.getProperty("user.dir");
        BufferedReader br;
        try {
            br = new BufferedReader(new InputStreamReader(new FileInputStream(url + skFileUrl), "UTF-8"));
            String tempStr;
            while ((tempStr = br.readLine()) != null) {
                listStr.add(tempStr);
            }
        } catch (Exception e) {
            return listStr;
        }
        return listStr;
    }

    /**
     * 机器人测试
     */
    public void jqr(JsonDataDto jsonDataDto, XBMqttDataUtil xbMqttDataUtil) {
        try {

            long millis = System.currentTimeMillis() / 1000;
            //解析sk证书信息
            Integer version = resolveSk(jsonDataDto.getSkurl());
            //reportDto.put("productKey", "PaintRobot");
            //解析设备txt文本获取静态属性值
            Map<String, Object> reportDto = new LinkedHashMap<>();
            Map<String, Object> fileSitesMap = readTxtFileToMap(jsonDataDto.getTxtFileUrl());
            Map<String, Object> sites = new HashMap<>();
            if (!fileSitesMap.isEmpty()) {
                reportDto.put("cmdId", jsonDataDto.getCmdId());//指 令 标 识
                reportDto.put("version", version);//版本
                reportDto.put("sequenceId", jsonDataDto.getSequenceId());//必填，值为1
                reportDto.put("edgeTime", millis);
                reportDto.put("productKey", jsonDataDto.getProductKey()); //必填，表示设备类型，设备类型在IoT平台上维护
                reportDto.put("clientUuid", jsonDataDto.getClientUuid());  //TODO 上报到客户那边数据中必要的参数值

                sites.put("DeviceID", fileSitesMap.get("设备ID"));
                sites.put("CompanyNo", fileSitesMap.get("公司编码"));
                sites.put("factoryNo", fileSitesMap.get("工厂编码"));
                //sites.put("AssetNo","");
                sites.put("DeviceName", fileSitesMap.get("设备名称"));
                sites.put("DeviceType", fileSitesMap.get("设备类型"));
                //sites.put("CncType","");
            }
            //1 redis和mqtt分开
            if (separate == 1) {
                //TODO 同时MQTT上报和推送前端数据
                Map<String, Object> objectMap = base64decrypt.jqrSendString(jsonDataDto, sites, millis, reportDto);
                String str = JSONObject.toJSONString(objectMap, SerializerFeature.WriteMapNullValue);
                System.out.println("IOT上报数据：" + str);
                xbMqttDataUtil.publish(str);
            } else if (separate == 2) {
                // TODO  只推送前端数据
                Map<String, Object> objectMap = base64decrypt.jqrSendString(jsonDataDto, sites, millis, reportDto);
                StaticData.data.put(jsonDataDto.getMongdbTableb(), objectMap);
            } else if (separate == 3) {
                //TODO  只发送mqtt
                Map<String, Object> objectMap = base64decrypt.jqrSendString(jsonDataDto, sites, millis, reportDto);
                String str = JSONObject.toJSONString(objectMap, SerializerFeature.WriteMapNullValue);
                xbMqttDataUtil.publish(str);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
