package com.jln.system.web.aop;

import com.alibaba.fastjson.JSON;
import com.jln.common.entity.Result;
import com.jln.common.utils.HttpClientUtil;
import com.jln.common.utils.StringUtils;
import com.jln.system.web.action.sm.user.entity.SysEmployee;
import com.jln.system.web.action.sm.user.service.SysEmployeeService;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;


/*@Aspect
@Component*/
public class HttpLogAspect {

    @Value("${systemm.anagement.address}")
    private String URL;

    private static final Logger log = LoggerFactory.getLogger(HttpLogAspect.class);
    @Autowired
    private SysEmployeeService sysEmployeeService;

    @Pointcut("execution(* com.jln.system.web.action.*.*.controller..*(..))")
    public void log() {
    }

    @Before("log()")
    public void doBefore(JoinPoint joinPoint) {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest httpServletRequest = attributes.getRequest();
        String token = httpServletRequest.getHeader("api_key");
        if(!StringUtils.isBlank(token)){
            synchronization(token);
        }
        //获取请求地址
        log.info("url={}", httpServletRequest.getRequestURL());

        //请求方法
        log.info("method={}", httpServletRequest.getMethod());

        //类方法
        log.info("class_method={}", joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName());
        //
        // 获取参数名
        String[] argNames = ((MethodSignature)joinPoint.getSignature()).getParameterNames();
        // 获取参数值ֵ
        Object[] args = joinPoint.getArgs();
        for (int i = 0; i < args.length; i++) {
        }
        log.info("args={}", joinPoint.getArgs());
    }

    @AfterReturning(returning = "object", pointcut = "log()")
    public void doAfterReturning(Object object) {
        if(object==null)
        {

        }else {
            log.info("response={}", object.toString());
        }

    }

    /**
     * 同步用户数据，存在则不同步
     * @param token
     */
    public void synchronization(String token){

    }


    private void addSysEmployee(String token,String uid){
      /*  HttpHeaders headers = new HttpHeaders();
        headers.add("api_key",token);*/
        HashMap<String, String> map = new HashMap<>();
        map.put("oid",uid);
        String reponse = HttpClientUtil.doGet(URL,map,token);
        Result result = JSON.parseObject(reponse, Result.class);
        if(!result.confirmSuccess()){
            log.info(result.getMsg());
            return;
        }
        com.jln.system.web.utils.SysEmployee object = JSON.parseObject(JSON.toJSONString(result.getData()), com.jln.system.web.utils.SysEmployee.class);
        SysEmployee sysEmployee = new SysEmployee();
        sysEmployee.setEmployeeOid(object.getSysEmployeeOid());
        sysEmployee.setEmployeeName(object.getEmployeeName());
        sysEmployee.setPhone(object.getPhone());
        sysEmployee.setCodeing(object.getEmployeeCode());
        sysEmployee.setJobNumber(object.getEmployeeCode());
        sysEmployee.setIdCard(object.getIdCard());
        sysEmployee.setNativePlace(object.getThePlace());
        sysEmployee.setEntryTime(object.getDataOfEntry());
        sysEmployee.setBlockUp("0");
        sysEmployee.setDel(object.getDel());
        if(sysEmployeeService.save(sysEmployee))
        {
            log.info("用户信息同步成功");
        }else
        {
            log.info("用户信息同步失败");
        }


    }
}
