package com.jln.system.web.interceptor;

import com.jln.common.annotation.PermissionAop;
import com.jln.common.entity.UserID;
import com.jln.system.web.utils.Actionrule;
import com.jln.system.web.utils.ApplicationContextHelper;
import com.jln.system.web.utils.StringUtils;
import lombok.experimental.Accessors;
import net.sf.jsqlparser.JSQLParserException;
import net.sf.jsqlparser.expression.operators.conditional.AndExpression;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.select.Join;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SubSelect;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.SqlSource;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

/**
 * @author 86180
 * @version : MyInterceptor
 */
@Accessors(chain = true)
@Intercepts(
        @Signature(method = "query",
                type = Executor.class,
                args = {MappedStatement.class, Object.class, RowBounds.class, ResultHandler.class}
        )
)
public class MyInterceptor implements Interceptor {
    /**
     * @param invocation { 代理对象，被监控方法对象，当前被监控方法运行时需要的实参 }
     * @return
     * @throws Throwable
     */

    private ApplicationContext beanFactory;

    private UserID userID;

    Logger LOGGER = LoggerFactory.getLogger(MyInterceptor.class);

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        this.loadService();

        Object[] args = invocation.getArgs();
        MappedStatement statement = (MappedStatement) args[0];
        //获取了sql执行所需的参数
        Object parameterObject = args[1];
        BoundSql boundSql = statement.getBoundSql(parameterObject);
        Class<?> classType = Class.forName(statement.getId().substring(0, statement.getId().lastIndexOf(".")));
        PermissionAop annotation = classType.getAnnotation(PermissionAop.class);
        if (annotation != null) {
            return invocation.proceed();
        }
        //从预编译中取出的sql
        String sql = boundSql.getSql();
        String replace = processSelectSql(sql);
        // 改变sql语句
        BoundSql newBoundSql = new BoundSql(statement.getConfiguration(), replace, boundSql.getParameterMappings(), boundSql.getParameterObject());
        MappedStatement newMs = copyFromMappedStatement(statement, new BoundSqlSqlSource(newBoundSql));
        //将新的sql装入到执行器中  如果是批量查询将导致参数的重复循环
        invocation.getArgs()[0] = newMs;
        return invocation.proceed();
    }

    /**
     * @param target 表示被拦截的对象，此处为 Executor 的实例对象
     *               作用：如果被拦截对象所在的类有实现接口，就为当前拦截对象生成一个代理对象
     *               如果被拦截对象所在的类没有指定接口，这个对象之后的行为就不会被代理操作
     * @return
     */
    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {

    }

    /**
     * 判断
     *
     * @param sql
     * @return
     * @throws JSQLParserException
     */
    public String processSelectSql(String sql) throws JSQLParserException {
        LOGGER.info("sql语句:" + sql);
        String replaceSql = null;
        Select select = null;
        try {
            select = (Select) CCJSqlParserUtil.parse(sql);
            PlainSelect selectBody = (PlainSelect) select.getSelectBody();
            String mainTable = null;
            if (selectBody.getFromItem() instanceof Table) {
                // 获取主表名称
                mainTable = ((Table) selectBody.getFromItem()).getName().replace("`", "");
            } else if (selectBody.getFromItem() instanceof SubSelect) {
                replaceSql = processSelectSql(((SubSelect) selectBody.getFromItem()).getSelectBody().toString());
            }
            if (!StringUtils.isEmpty(replaceSql)) {
                sql = sql.replace(((SubSelect) selectBody.getFromItem()).getSelectBody().toString(), replaceSql);
            }
            String condExpr = null;
            // 主体添加限制
            List<Actionrule> manacles = getroleactoinList(mainTable);
            List<List<Actionrule>> lists = new ArrayList<>();
            if (!StringUtils.isEmpty(selectBody.getJoins())) {
                for (Join jointable : selectBody.getJoins()) {
                    String jointablename = ((Table) jointable.getRightItem()).getName();
                    if (getroleactoinList(jointablename) == null) {
                        continue;
                    }
                    lists.add(getroleactoinList(jointablename));
                }
            }
            if (manacles == null && lists.size() == 0) {
                return sql;
            }
            if (manacles != null) {
                for (Actionrule actionrule : manacles) {
                    condExpr = mainTable + "." + actionrule.getColumnName() + actionrule.getOperation() + " '" + actionrule.getValue() + "'";
                    if (selectBody.getWhere() == null) {
                        selectBody.setWhere(CCJSqlParserUtil.parseCondExpression(condExpr));
                    } else {
                        AndExpression and = new AndExpression(selectBody.getWhere(), CCJSqlParserUtil.parseCondExpression(condExpr));
                        selectBody.setWhere(and);
                    }
                }
            }
            if (lists.size() != 0) {
                // 获取join表信息
                for (Join jointable : selectBody.getJoins()) {
                    String jointablename = ((Table) jointable.getRightItem()).getName();
                    List<Actionrule> actionrules = getroleactoinList(jointablename);
                    if (actionrules == null) {
                        continue;
                    }
                    for (Actionrule actionrule : actionrules) {
                        condExpr = mainTable + "." + actionrule.getColumnName() + actionrule.getOperation() + "'" + actionrule.getValue() + "'";
                        if (selectBody.getWhere() == null) {
                            selectBody.setWhere(CCJSqlParserUtil.parseCondExpression(condExpr));
                        } else {
                            AndExpression and = new AndExpression(selectBody.getWhere(), CCJSqlParserUtil.parseCondExpression(condExpr));
                            selectBody.setWhere(and);
                        }
                    }
                }
            }

            if (sql.indexOf("limit ?,?") != -1 && select.toString().indexOf("LIMIT ? OFFSET ?") != -1) {
                sql = select.toString().replace("LIMIT ? OFFSET ?", "limit ?,?");
            } else {
                sql = select.toString();
            }
        } catch (JSQLParserException e) {
            e.printStackTrace();
        }
        return sql;
    }

    public List<Actionrule> getroleactoinList(String tablename) {
        return null;
    }

    private MappedStatement copyFromMappedStatement(MappedStatement ms, SqlSource newSqlSource) {
        MappedStatement.Builder builder = new MappedStatement.Builder(ms.getConfiguration(), ms.getId(), newSqlSource, ms.getSqlCommandType());
        builder.resource(ms.getResource());
        builder.fetchSize(ms.getFetchSize());
        builder.statementType(ms.getStatementType());
        builder.keyGenerator(ms.getKeyGenerator());
        if (ms.getKeyProperties() != null && ms.getKeyProperties().length > 0) {
            builder.keyProperty(ms.getKeyProperties()[0]);
        }
        builder.timeout(ms.getTimeout());
        builder.parameterMap(ms.getParameterMap());
        builder.resultMaps(ms.getResultMaps());
        builder.resultSetType(ms.getResultSetType());
        builder.cache(ms.getCache());
        builder.flushCacheRequired(ms.isFlushCacheRequired());
        builder.useCache(ms.isUseCache());
        return builder.build();
    }

    public static class BoundSqlSqlSource implements SqlSource {
        private BoundSql boundSql;

        public BoundSqlSqlSource(BoundSql boundSql) {
            this.boundSql = boundSql;
        }

        @Override
        public BoundSql getBoundSql(Object parameterObject) {
            return boundSql;
        }
    }

    private void loadService() {
        if (null == beanFactory) {
            beanFactory = ApplicationContextHelper.getApplicationContext();
            if (null == beanFactory) {
                return;
            }
        }
        if (userID == null) {
            userID = beanFactory.getBean(UserID.class);
        }
    }
}
