package com.jln.system.web.utils;

import com.baomidou.mybatisplus.annotation.TableLogic;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.jln.common.constant.LocalDateTimeConst;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;

/**
 * <p>
 * 员工表
 * </p>
 *
 * @author 
 * @since 2020-04-28
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value="SysEmployee对象", description="员工表")
public class SysEmployee{

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "员工表主键",example = "0f10698e-fcc4-4033-9aa1-02167a24f7a1")
    private Long sysEmployeeOid;

    @NotBlank(message = "员工工号不能为空")
    @ApiModelProperty(value = "员工编码，工号",example = "10002")
    private String employeeCode;

    @NotBlank(message = "员工名称不能为空")
    @ApiModelProperty(value = "员工名称",example = "张三")
    private String employeeName;

    @ApiModelProperty(value = "工作电话",example = "15982235329")
    private String workPhoneNumber;

    @ApiModelProperty(value = "移动电话",example = "15982235329")
    private String phone;

    @ApiModelProperty(value = "邮箱",example = "sandaoLgc@163.com")
    private String email;

    @ApiModelProperty(value = "身份证号",example = "513021199606231456")
    private String idCard;

    @ApiModelProperty(value = "籍贯",example = "四川-达州")
    private String thePlace;

    @ApiModelProperty(value = "性别，1男，0女",example = "1")
    private Integer sex;

    @ApiModelProperty(value = "出生日期",example = "1998-02-12")
    private String birthday;

    @ApiModelProperty(value = "用户外键",example = "")
    private String userFk;

    @ApiModelProperty(value = "入职日期",example = "2019-12-15 12:00:00")
    @JsonFormat(pattern = LocalDateTimeConst.DATE_TIME_PATTERN)
    private LocalDateTime dataOfEntry;

    @NotNull(message = "婚姻状况不能为空")
    @Min(value = 0,message = "婚姻状况不能为空")
    @Max(value = 1,message = "婚姻状况不能为空")
    @ApiModelProperty(value = "婚姻状况，0已婚，1未婚",example = "0")
    private Integer mamageSta;

    @ApiModelProperty(value = "银行账号",example = "62011504147586")
    private String bankNumber;

    @ApiModelProperty(value = "照片，放URL地址",example = "http://192.168.0.114:8010/mes/imges/1.jpg")
    private String photo;

    @ApiModelProperty(value = "备注",example = "测试数据")
    private String bz;

    @ApiModelProperty(value = "部门外键",example = "部门外键")
    private String organiceFk;

    @TableLogic
    @ApiModelProperty(value = "是否删除，0没有，1删除",hidden = true)
    private Integer del;


}
